using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Printing;

namespace TYWinforms.Day7 {
   public class Exercise1 : Form {
      PageSettings objPageSettings = new PageSettings();
      MainMenu mnuMain = new MainMenu();
      MenuItem mniFile = new MenuItem("File");
      MenuItem mniOpen = new MenuItem("Open");
      MenuItem mniSave = new MenuItem("Save");
      MenuItem mniPageSetup = new MenuItem("Page Setup");
      MenuItem mniPrintPreview = new MenuItem("Print Preview");
      MenuItem mniPrint = new MenuItem("Print");
      MenuItem mniEdit = new MenuItem("Edit");
      MenuItem mniFont = new MenuItem("Font");
      MenuItem mniColor = new MenuItem("Color");

      HelpDialog dlgHelp = new HelpDialog();

      public Exercise1() {
         mnuMain.MenuItems.Add(mniFile);
         mnuMain.MenuItems.Add(mniEdit);

         mniFile.MenuItems.Add(mniOpen);
         mniFile.MenuItems.Add(mniSave);
         mniFile.MenuItems.Add("-");
         mniFile.MenuItems.Add(mniPageSetup);
         mniFile.MenuItems.Add(mniPrintPreview);
         mniFile.MenuItems.Add(mniPrint);

         mniEdit.MenuItems.Add(mniFont);
         mniEdit.MenuItems.Add(mniColor);

         mniOpen.Click += new EventHandler(this.FileMenuClick);
         mniSave.Click += new EventHandler(this.FileMenuClick);
         mniPageSetup.Click += new EventHandler(this.FileMenuClick);
         mniPrintPreview.Click += new EventHandler(this.FileMenuClick);
         mniPrint.Click += new EventHandler(this.FileMenuClick);
         mniFont.Click += new EventHandler(this.EditMenuClick);
         mniColor.Click += new EventHandler(this.EditMenuClick);

         mniOpen.Select += new EventHandler(this.DisplayHelp);
         mniSave.Select += new EventHandler(this.DisplayHelp);
         mniPageSetup.Select += new EventHandler(this.DisplayHelp);
         mniPrintPreview.Select += new EventHandler(this.DisplayHelp);
         mniPrint.Select += new EventHandler(this.DisplayHelp);
         mniFont.Select += new EventHandler(this.DisplayHelp);
         mniColor.Select += new EventHandler(this.DisplayHelp);

         this.Menu = mnuMain;
         this.Text = "Exercise 1";
      }

      public void FileMenuClick(Object Sender, EventArgs e) {
         MenuItem mniTemp = (MenuItem)Sender;

         switch (mniTemp.Text) {
            case "Open":
               OpenFileDialog dlgOpen = new OpenFileDialog();
               if (dlgOpen.ShowDialog() == DialogResult.OK) {
                  //open file
               }
               break;
            case "Save":
               SaveFileDialog dlgSave = new SaveFileDialog();
               if (dlgSave.ShowDialog() == DialogResult.OK) {
                  //save file
               }
               break;
            case "Page Setup":
               PageSetupDialog dlgPageSetup = new PageSetupDialog();
               dlgPageSetup.PageSettings = objPageSettings;
               dlgPageSetup.ShowDialog();
               break;
            case "Print Preview":
               PrintPreviewDialog dlgPrintPreview = new PrintPreviewDialog();
               dlgPrintPreview.Document = new PrintDocument();
               dlgPrintPreview.ShowDialog();
               break;
            case "Print":
               PrintDialog dlgPrint = new PrintDialog();
               dlgPrint.PrinterSettings = new PrinterSettings();
               if (dlgPrint.ShowDialog() == DialogResult.OK) {
                  //print
               }
               break;
         }
      }

      public void EditMenuClick(Object Sender, EventArgs e) {
         MenuItem mniTemp = (MenuItem)Sender;

         switch (mniTemp.Text) {
            case "Font":
               FontDialog dlgFont = new FontDialog();
               if (dlgFont.ShowDialog() == DialogResult.OK) {
                  this.Font = dlgFont.Font;
               }
               break;
            case "Color":
               ColorDialog dlgColor = new ColorDialog();
               if (dlgColor.ShowDialog() == DialogResult.OK) {
                  this.BackColor = dlgColor.Color;
               }
               break;
         }
      }

      public void DisplayHelp(Object Sender, EventArgs e) {
         MenuItem mniTemp = (MenuItem)Sender;

         if (!dlgHelp.Visible) { dlgHelp.Show(); }

         switch (mniTemp.Text) {
            case "Open":
               dlgHelp.HelpText = "Open a file";
               break;
            case "Save":
               dlgHelp.HelpText = "Save this file";
               break;
            case "Page Setup":
               dlgHelp.HelpText = "Change page settings";
               break;
            case "Print Preview":
               dlgHelp.HelpText = "Preview the document before printing";
               break;
            case "Print":
               dlgHelp.HelpText = "Print the current document";
               break;
            case "Font":
               dlgHelp.HelpText = "Change the current font face, color, size, etc";
               break;
            case "Color":
               dlgHelp.HelpText = "Change the background color of the application";
               break;
         }
      }

      public static void Main() {
         Application.Run(new Exercise1());
      }
   }

   public class HelpDialog : Form {
      private Label lblMessage = new Label();

      public String HelpText {
         get { return lblMessage.Text; }
         set { lblMessage.Text = value; }
      }

      public HelpDialog() {
         lblMessage.Size = new Size(100,100);
         lblMessage.Text = "Help screen";

         this.Controls.Add(lblMessage);
         this.BackColor = Color.Yellow;
         this.Size = new Size(100,100);
         this.FormBorderStyle = FormBorderStyle.FixedToolWindow;
      }
   }
}