using System;
using System.Windows.Forms;
using System.Drawing;

namespace TYWinForms.Day17 {

   public class Exercise : Form {
      private MainMenu mnuCustom = new MainMenu();
      private MenuItem miCustom = new MenuItem();
      private MenuItem miOne = new MenuItem();
      private MenuItem miTwo = new MenuItem();
      private MenuItem miThree = new MenuItem();
      private MenuItem miFour = new MenuItem();

      private Label lblHighlight = new Label();
      private Label lblActiveText = new Label();
      private Label lblBackground = new Label();
      private Label lblRegText = new Label();

      private ComboBox cboHighlight = new ComboBox();
      private ComboBox cboActiveText = new ComboBox();
      private ComboBox cboBackground = new ComboBox();
      private ComboBox cboRegText = new ComboBox();

      private Brush brsHighlight;
      private Brush brsActiveText;
      private Brush brsBackground;
      private Brush brsRegText;

      private SolidBrush[] arrBrushes = new SolidBrush[10];

      public Exercise() {
         arrBrushes[0] = (SolidBrush)Brushes.White;
         arrBrushes[1] = (SolidBrush)Brushes.Red;
         arrBrushes[2] = (SolidBrush)Brushes.Green;
         arrBrushes[3] = (SolidBrush)Brushes.Blue;
         arrBrushes[4] = (SolidBrush)Brushes.LightGray;
         arrBrushes[5] = (SolidBrush)Brushes.Orange;
         arrBrushes[6] = (SolidBrush)Brushes.Yellow;
         arrBrushes[7] = (SolidBrush)Brushes.Black;
         arrBrushes[8] = (SolidBrush)Brushes.BlanchedAlmond;
         arrBrushes[9] = (SolidBrush)Brushes.DarkSalmon;
 
         brsHighlight = Brushes.Blue;
         brsActiveText = Brushes.White;
         brsBackground = Brushes.LightGray;
         brsRegText = Brushes.Black;

         miCustom = mnuCustom.MenuItems.Add("Custom Menu");
         miOne = miCustom.MenuItems.Add("Item 1");
         miTwo = miCustom.MenuItems.Add("Item 2");
         miThree = miCustom.MenuItems.Add("Item 3");
         miFour = miCustom.MenuItems.Add("Item 4");

         miCustom.OwnerDraw = true;
         miCustom.DrawItem += new DrawItemEventHandler(this.DrawTopLevel);
         miCustom.MeasureItem += new MeasureItemEventHandler(this.MeasureItem);

         miOne.OwnerDraw = true;
         miOne.DrawItem += new DrawItemEventHandler(this.DrawItem);
         miOne.MeasureItem += new MeasureItemEventHandler(this.MeasureItem);

         miTwo.OwnerDraw = true;
         miTwo.DrawItem += new DrawItemEventHandler(this.DrawItem);
         miTwo.MeasureItem += new MeasureItemEventHandler(this.MeasureItem);

         miThree.OwnerDraw = true;
         miThree.DrawItem += new DrawItemEventHandler(this.DrawItem);
         miThree.MeasureItem += new MeasureItemEventHandler(this.MeasureItem);

         miFour.OwnerDraw = true;
         miFour.DrawItem += new DrawItemEventHandler(this.DrawItem);
         miFour.MeasureItem += new MeasureItemEventHandler(this.MeasureItem);

         lblHighlight.Text = "Highlight color:";
         lblHighlight.Width = 80;
         lblHighlight.Location = new Point(0,10);

         lblActiveText.Text = "Active Text color:";
         lblActiveText.Size = new Size(80,30);
         lblActiveText.Location = new Point(0,50);

         lblBackground.Text = "Background color:";
         lblBackground.Size = new Size(80,30);
         lblBackground.Location = new Point(0,90);

         lblRegText.Text = "Regular Text color:";
         lblRegText.Size = new Size(80,30);
         lblRegText.Location = new Point(0,130);

         cboHighlight.DataSource = arrBrushes.Clone();
         cboHighlight.Location = new Point(90,10);
         cboHighlight.Width = 200;
         cboHighlight.DisplayMember = "Color";
         cboHighlight.DrawMode = DrawMode.OwnerDrawFixed;
         cboHighlight.DrawItem += new DrawItemEventHandler(this.DrawBox);
         cboHighlight.SelectedValueChanged += new EventHandler(this.ChangeHighLightColor);

         cboActiveText.DataSource = arrBrushes.Clone();
         cboActiveText.Location = new Point(90,50);
         cboActiveText.Width = 200;
         cboActiveText.DisplayMember = "Color";
         cboActiveText.DrawMode = DrawMode.OwnerDrawFixed;
         cboActiveText.DrawItem += new DrawItemEventHandler(this.DrawBox);
         cboActiveText.SelectedValueChanged += new EventHandler(this.ChangeActiveTextColor);

         cboBackground.DataSource = arrBrushes.Clone();
         cboBackground.Location = new Point(90,90);
         cboBackground.Width = 200;
         cboBackground.DisplayMember = "Color";
         cboBackground.DrawMode = DrawMode.OwnerDrawFixed;
         cboBackground.DrawItem += new DrawItemEventHandler(this.DrawBox);
         cboBackground.SelectedValueChanged += new EventHandler(this.ChangeBackgroundColor);

         cboRegText.DataSource = arrBrushes.Clone();
         cboRegText.Location = new Point(90,130);
         cboRegText.Width = 200;
         cboRegText.DisplayMember = "Color";
         cboRegText.DrawMode = DrawMode.OwnerDrawFixed;
         cboRegText.DrawItem += new DrawItemEventHandler(this.DrawBox);
         cboRegText.SelectedValueChanged += new EventHandler(this.ChangeRegTextColor);

         this.Text = "Exercise";
         this.Menu = mnuCustom;
         this.Controls.Add(lblHighlight);
         this.Controls.Add(lblActiveText);
         this.Controls.Add(lblBackground);
         this.Controls.Add(lblRegText);
         this.Controls.Add(cboHighlight);
         this.Controls.Add(cboActiveText);
         this.Controls.Add(cboBackground);
         this.Controls.Add(cboRegText);
      }

      private void DrawBox(Object Sender, DrawItemEventArgs e) {
         e.DrawBackground();

         if ((e.State & DrawItemState.Selected) == DrawItemState.Selected) { 
            e.Graphics.FillRectangle(arrBrushes[e.Index], new Rectangle(e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height));
            e.Graphics.DrawString(((SolidBrush)cboHighlight.Items[e.Index]).Color.Name, new Font("Arial",10), ((SolidBrush)cboHighlight.Items[9-e.Index]), new Point(e.Bounds.X, e.Bounds.Y));
         } else {
            e.Graphics.DrawString(((SolidBrush)cboHighlight.Items[e.Index]).Color.Name, new Font("Arial",10), ((SolidBrush)cboHighlight.Items[e.Index]), new Point(e.Bounds.X, e.Bounds.Y));
         }      
      }

      private void ChangeHighLightColor(Object Sender, EventArgs e) {
         brsHighlight = (SolidBrush)((ComboBox)Sender).SelectedValue;
      }

      private void ChangeActiveTextColor(Object Sender, EventArgs e) {
         brsActiveText = (SolidBrush)((ComboBox)Sender).SelectedValue;
      }

      private void ChangeBackgroundColor(Object Sender, EventArgs e) {
         brsBackground = (SolidBrush)((ComboBox)Sender).SelectedValue;
      }

      private void ChangeRegTextColor(Object Sender, EventArgs e) {
         brsRegText = (SolidBrush)((ComboBox)Sender).SelectedValue;
      }

      private void MeasureItem(Object Sender, MeasureItemEventArgs e) {
         SizeF stringSize = e.Graphics.MeasureString(((MenuItem)Sender).Text, this.Font);

         e.ItemHeight = (int)stringSize.Height + 6;
         e.ItemWidth = (int)stringSize.Width + 10;
      }

      private void DrawTopLevel(Object Sender, DrawItemEventArgs e) {
         if (((e.State & DrawItemState.HotLight) == DrawItemState.HotLight) | (e.State & DrawItemState.Selected) == DrawItemState.Selected) { 
            e.Graphics.DrawString(((MenuItem)Sender).Text, new Font("Times New Roman",12), Brushes.Blue, new Point(e.Bounds.X, e.Bounds.Y));
         } else {
            e.Graphics.DrawString(((MenuItem)Sender).Text, new Font("Times New Roman",12), Brushes.Black, new Point(e.Bounds.X, e.Bounds.Y));
         }
      }

      private void DrawItem(Object Sender, DrawItemEventArgs e) {
         e.Graphics.FillRectangle(brsBackground, new Rectangle(e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height));

         if (((e.State & DrawItemState.HotLight) == DrawItemState.HotLight) | (e.State & DrawItemState.Selected) == DrawItemState.Selected) { 
            e.Graphics.FillRectangle(brsHighlight, new Rectangle(e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height));
            e.Graphics.DrawString(((MenuItem)Sender).Text, new Font("Times New Roman",12), brsActiveText, new Point(e.Bounds.X, e.Bounds.Y));
         } else {
            e.Graphics.DrawString(((MenuItem)Sender).Text, new Font("Times New Roman",12), brsRegText, new Point(e.Bounds.X, e.Bounds.Y));
         }
      }

      public static void Main() {
         Application.Run(new Exercise());
      }
   }

}