using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace TYWinforms.Day18.Controls {
   public class EyeControl : Control {
      private Point ptPosition;
      private Color clrIris;
      private int intBlood;
      private Color clrBlood;
      private System.Timers.Timer objTimer = new System.Timers.Timer();
      private EventHandler onEyeMoved;
      private GraphicsPath pthClip = new GraphicsPath();
      private Region regClip;

      public Point PupilPosition {
         get { return ptPosition; }
         set {
            ptPosition = value;
            Invalidate();
         }
      }

      public Color IrisColor {
         get { return clrIris; }
         set {
            clrIris = value;
            Invalidate();
         }
      }

      public int BloodShotedness {
         get { return intBlood; }
         set {
            intBlood = value;
            clrBlood = Color.FromArgb(255, 255 - (int)(255*(intBlood/100.0)), 255 - (int)(255*(intBlood/100.0)));
            Invalidate();
         }
      }

      public EyeControl() {
         ptPosition = new Point(0,0);
         intBlood = 0;
         clrBlood = Color.White;
         clrIris = Color.Blue;

         objTimer.Interval = 100;
         objTimer.Elapsed += new System.Timers.ElapsedEventHandler(this.Update);
         objTimer.Enabled = true;

         this.Size = new Size(100,100);
         this.HandleCreated += new EventHandler(this.SetBounds);
      }

      public event EventHandler EyeMoved {
         add { onEyeMoved += value; }
         remove { onEyeMoved -= value; }
      }

      private void SetBounds(Object Sender, EventArgs e) {
         pthClip.AddEllipse(ClientRectangle);
         regClip = new Region(pthClip);
      }

      private void Update(Object Sender, System.Timers.ElapsedEventArgs e) {
         ptPosition = Cursor.Position;

         ptPosition.X /= (Screen.PrimaryScreen.Bounds.Width / ClientRectangle.Width);
         ptPosition.Y /= (Screen.PrimaryScreen.Bounds.Height / ClientRectangle.Height);

         if (ptPosition.X > ClientRectangle.Width-30) {
            ptPosition.X = ClientRectangle.Width-30;
         }

         if (ptPosition.Y > ClientRectangle.Height-30) {
            ptPosition.Y = ClientRectangle.Height-30;
         }

         OnEyeMoved(EventArgs.Empty);
         Invalidate();
      }

      protected virtual void OnEyeMoved(EventArgs e) {
         if (onEyeMoved != null) {
            onEyeMoved(this, e);
         }
      }

      protected override void OnPaint(PaintEventArgs e) {
         e.Graphics.Clip = regClip;
         e.Graphics.FillEllipse(new SolidBrush(clrBlood), ClientRectangle);

         e.Graphics.FillEllipse(new SolidBrush(clrIris), new Rectangle(ptPosition.X, ptPosition.Y, 30, 30));
         e.Graphics.FillEllipse(Brushes.Black, new Rectangle(ptPosition.X+5, ptPosition.Y+5, 20, 20));
      }
   }
}