Imports System
Imports System.Windows.Forms
Imports System.Drawing
Imports System.Drawing.Printing
Imports System.ComponentModel

namespace TYWinforms.P1 
   public class NetWord : Inherits Form
      private rtbDocument as New RichTextBox()
      private cmnuDocument as New ContextMenu()
      private sbarMain as New StatusBar()
      private spnlLine as New StatusBarPanel()

      private mnuMain as New MainMenu()
      private mniFile as New MenuItem("File")
      private mniOpen as New MenuItem("Open...")
      private mniSave as New MenuItem("Save")
      private mniPageSetup as New MenuItem("Page Setup...")
      private mniPrintPreview as New MenuItem("Print Preview")
      private mniPrint as New MenuItem("Print...")
      private mniExit as New MenuItem("Exit")

      private mniEdit as New MenuItem("Edit")
      private mniUndo as New MenuItem("Undo")
      private mniCut as New MenuItem("Cut")
      private mniCopy as New MenuItem("Copy")
      private mniPaste as New MenuItem("Paste")
      private mniFind as New MenuItem("Find...")

      private mniFormat as New MenuItem("Format")
      private mniFont as New MenuItem("Font...")

      private fntCurrent as New Font("Times New Roman", 10)
      private fntColor = Color.Black
      private dlgFont as New FontDialog()
      private dlgOpen as New OpenFileDialog()
      private dlgSave as New SaveFileDialog()
      private dlgPrintPreview as New PrintPreviewDialog()
      private dlgPageSetup as New PageSetupDialog()
      private dlgPrint as New PrintDialog()

      private objPageSettings as New PageSettings()
      private objPrintSettings as New PrinterSettings()

      public sub New()
         rtbDocument.Dock = DockStyle.Fill
         rtbDocument.Font = fntCurrent
         rtbDocument.HideSelection = false
         rtbDocument.AcceptsTab = true
         AddHandler rtbDocument.SelectionChanged, New EventHandler(AddressOf Me.UpdateStatus)

         sbarMain.ShowPanels = true
         sbarMain.Panels.Add(spnlLine)
         spnlLine.AutoSize = StatusBarPanelAutoSize.Spring
         spnlLine.Alignment = HorizontalAlignment.Right

         AddHandler mniOpen.Click, New EventHandler(AddressOf Me.FileClicked)
         AddHandler mniSave.Click, New EventHandler(AddressOf Me.FileClicked)
         AddHandler mniPageSetup.Click, New EventHandler(AddressOf Me.FileClicked)
         AddHandler mniPrintPreview.Click, New EventHandler(AddressOf Me.FileClicked)
         AddHandler mniPrint.Click, New EventHandler(AddressOf Me.FileClicked)
         AddHandler mniExit.Click, New EventHandler(AddressOf Me.FileClicked)

         AddHandler mniUndo.Click, New EventHandler(AddressOf Me.EditClicked)
         AddHandler mniCut.Click, New EventHandler(AddressOf Me.EditClicked)
         AddHandler mniCopy.Click, New EventHandler(AddressOf Me.EditClicked)
         AddHandler mniPaste.Click, New EventHandler(AddressOf Me.EditClicked)
         AddHandler mniFind.Click, New EventHandler(AddressOf Me.EditClicked)

         AddHandler mniFont.Click, New EventHandler(AddressOf Me.FormatClicked)

         mnuMain.MenuItems.Add(mniFile)
         mnuMain.MenuItems.Add(mniEdit)
         mnuMain.MenuItems.Add(mniFormat)
         
         mniOpen.Shortcut = Shortcut.CtrlO
         mniSave.Shortcut = Shortcut.CtrlS
         mniPrint.Shortcut = Shortcut.CtrlP
         mniUndo.Shortcut = Shortcut.CtrlZ
         mniCut.Shortcut = Shortcut.CtrlX
         mniCopy.Shortcut = Shortcut.CtrlC
         mniPaste.Shortcut = Shortcut.CtrlV
         mniFind.Shortcut = Shortcut.CtrlF

         mniOpen.ShowShortcut = true
         mniSave.ShowShortcut = true
         mniPrint.ShowShortcut = true
         mniUndo.ShowShortcut = true
         mniCut.ShowShortcut = true
         mniCopy.ShowShortcut = true
         mniPaste.ShowShortcut = true
         mniFind.ShowShortcut = true

         mniFile.MenuItems.Add(mniOpen)
         mniFile.MenuItems.Add(mniSave)
         mniFile.MenuItems.Add("-")
         mniFile.MenuItems.Add(mniPageSetup)
         mniFile.MenuItems.Add(mniPrintPreview)
         mniFile.MenuItems.Add(mniPrint)
         mniFile.MenuItems.Add("-")
         mniFile.MenuItems.Add(mniExit)

         mniEdit.MenuItems.Add(mniUndo)
         mniEdit.MenuItems.Add("-")
         mniEdit.MenuItems.Add(mniCut)
         mniEdit.MenuItems.Add(mniCopy)
         mniEdit.MenuItems.Add(mniPaste)
         mniEdit.MenuItems.Add("-")
         mniEdit.MenuItems.Add(mniFind)

         mniFormat.MenuItems.Add(mniFont)

         cmnuDocument.MenuItems.Add(mniCut.CloneMenu())
         cmnuDocument.MenuItems.Add(mniCopy.CloneMenu())
         cmnuDocument.MenuItems.Add(mniPaste.CloneMenu())
         cmnuDocument.MenuItems.Add("-")
         cmnuDocument.MenuItems.Add(mniFont.CloneMenu())
         AddHandler cmnuDocument.Popup, New EventHandler(AddressOf Me.HandleContext)
         rtbDocument.ContextMenu = cmnuDocument

         Me.Text = "NetWord v.1"
         Me.Name = "NetWord"
         Me.Size = new Size(800,600)
         Me.Menu = mnuMain
         AddHandler Me.Closing, New CancelEventHandler(AddressOf Me.DocumentClosing)         

         Me.Controls.Add(rtbDocument)
         Me.Controls.Add(sbarMain)
      End Sub

      private sub UpdateStatus(Sender as Object, e as EventArgs)
         dim text as String = rtbDocument.Text
         dim i, intNext as Integer
         dim strChar as String

         dim intLine as Integer = rtbDocument.GetLineFromCharIndex(rtbDocument.SelectionStart) + 1

         i = rtbDocument.SelectionStart
         while rtbDocument.GetPositionFromCharIndex(i).X > 1
            i = i - 1
         end while
         strChar = (rtbDocument.SelectionStart - i + 1).ToString()

         spnlLine.Text = "Line " & intLine.ToString() & " Char " & strChar
      end sub

      private sub HandleContext(Sender as Object, e as EventArgs)
         if rtbDocument.SelectionLength = 0 then
            cmnuDocument.MenuItems(0).Enabled = false
            cmnuDocument.MenuItems(1).Enabled = false
         else
            cmnuDocument.MenuItems(0).Enabled = true
            cmnuDocument.MenuItems(1).Enabled = true
         end if
      end sub

      private sub FileClicked(Sender as Object, e as EventArgs)
         dim mniTemp as MenuItem = CType(Sender, MenuItem)

         select case mniTemp.Text
            case "Open..."
               if dlgOpen.ShowDialog() = DialogResult.OK then
                  'open file
               end if
            case "Save"
               if dlgSave.ShowDialog() = DialogResult.OK then
                  'save file
               end if
            case "Page Setup..."
               dlgPageSetup.PageSettings = objPageSettings
               dlgPageSetup.ShowDialog()
            case "Print Preview"
               dlgPrintPreview.Document = new PrintDocument
               dlgPrintPreview.ShowDialog()
            case "Print..."
               dlgPrint.PrinterSettings = New PrinterSettings()
               if dlgPrint.ShowDialog() = DialogResult.OK then
                  'print
               end if
            case "Exit":
               Me.Close()
         end select
      end sub

      private sub EditClicked(Sender as Object, e as EventArgs)
         dim mniTemp as MenuItem = CType(Sender, MenuItem)

         select case mniTemp.Text
            case "Undo"
               rtbDocument.Undo()
            case "Cut"
               if rtbDocument.SelectedRtf <> "" then
                  rtbDocument.Cut()
               end if
            case "Copy"
               if rtbDocument.SelectedRtf <> "" then
                  rtbDocument.Copy()
               end if
            case "Paste"
               rtbDocument.Paste()
            case "Find..."
               dim dlgFind as New FindDialog(rtbDocument)
               dlgFind.Show()
         end select
      end sub

      private sub FormatClicked(Sender as Object, e as EventArgs)
         dim mniTemp as MenuItem = CType(Sender, MenuItem)

         select case mniTemp.Text
            case "Font..."
               dlgFont.ShowColor = true
               if dlgFont.ShowDialog() = DialogResult.OK then
                  fntCurrent = dlgFont.Font
                  fntColor = dlgFont.Color
                  rtbDocument.SelectionFont = fntCurrent
                  rtbDocument.SelectionColor = fntColor
               end if
         end select
      end sub

      private sub DocumentClosing(Sender as Object, e as CancelEventArgs)
         if rtbDocument.Modified then
            dim dr as DialogResult = MessageBox.Show("Do you want to save the changes?", Me.Name, MessageBoxButtons.YesNoCancel)

            if dr = DialogResult.Yes then
               MessageBox.Show("Too bad!", Me.Name)
            elseif dr = DialogResult.Cancel then
               e.Cancel = true
            end if
         end if
      end sub

      public shared sub Main()
         Application.Run(new NetWord())
      end sub
   end class
end namespace