using System;
using System.Windows.Forms;
using System.IO;
using System.Drawing;
using System.Drawing.Printing;
using System.ComponentModel;
using System.Collections;

namespace TYWinforms.P3 {
   public class Document : Form {
      public ArrayList arrPages = new ArrayList();
      public int intCurrentPage;

      private Panel pnlView = new Panel();
      private ContextMenu cmnuDocument = new ContextMenu();
      private StatusBar sbarMain = new StatusBar();
      private StatusBarPanel spnlLine = new StatusBarPanel();
      private VScrollBar scrlbar = new VScrollBar();

      private MainMenu mnuMain = new MainMenu();

      private MenuItem mniFile = new MenuItem("File");
      private MenuItem mniSave = new MenuItem("Save");
      private MenuItem mniPageSetup = new MenuItem("Page Setup...");
      private MenuItem mniPrintPreview = new MenuItem("Print Preview");
      private MenuItem mniPrint = new MenuItem("Print...");
      private MenuItem mniEdit = new MenuItem("Edit");
      private MenuItem mniUndo = new MenuItem("Undo");
      private MenuItem mniCut = new MenuItem("Cut");
      private MenuItem mniCopy = new MenuItem("Copy");
      private MenuItem mniPaste = new MenuItem("Paste");
      private MenuItem mniFind = new MenuItem("Find...");
      private MenuItem mniFormat = new MenuItem("Format");
      private MenuItem mniFont = new MenuItem("Font...");

      private Font fntCurrent = new Font("Times New Roman", 10);
      private Color fntColor = Color.Black;
      private FontDialog dlgFont = new FontDialog();
      private PageSetupDialog dlgPageSetup = new PageSetupDialog();

      private PageSettings objPageSettings = new PageSettings();
      private PrinterSettings objPrintSettings = new PrinterSettings();
      private StringReader objReader;

      public Document(string strName) {
         mniSave.Click += new EventHandler(this.FileClicked);
         mniPageSetup.Click += new EventHandler(this.FileClicked);
         mniPrintPreview.Click += new EventHandler(this.FileClicked);
         mniPrint.Click += new EventHandler(this.FileClicked);

         mnuMain.MenuItems.Add(mniFile);

         mniFile.MergeType = MenuMerge.MergeItems;
         mniFile.MenuItems.Add(mniSave);
         mniFile.MenuItems.Add("-");
         mniFile.MenuItems.Add(mniPageSetup);
         mniFile.MenuItems.Add(mniPrintPreview);
         mniFile.MenuItems.Add(mniPrint);

         mniFile.MergeOrder = 1;
         mniEdit.MergeOrder = 2;
         mniFormat.MergeOrder = 3;
         mniSave.MergeOrder = 30;
         mniFile.MenuItems[1].MergeOrder = 35;
         mniPageSetup.MergeOrder = 40;
         mniPrintPreview.MergeOrder = 45;
         mniPrint.MergeOrder = 50;

         sbarMain.ShowPanels = true;
         sbarMain.Font = new Font("Arial", 10);
         sbarMain.Panels.Add(spnlLine);
         spnlLine.AutoSize = StatusBarPanelAutoSize.Spring;
         spnlLine.Alignment = HorizontalAlignment.Right;

         mniUndo.Click += new EventHandler(this.EditClicked);
         mniCut.Click += new EventHandler(this.EditClicked);
         mniCopy.Click += new EventHandler(this.EditClicked);
         mniPaste.Click += new EventHandler(this.EditClicked);
         mniFind.Click += new EventHandler(this.EditClicked);

         mniFont.Click += new EventHandler(this.FormatClicked);

         mnuMain.MenuItems.Add(mniEdit);
         mnuMain.MenuItems.Add(mniFormat);
         
         mniUndo.ShowShortcut = true;
         mniCut.ShowShortcut = true;
         mniCopy.ShowShortcut = true;
         mniPaste.ShowShortcut = true;
         mniFind.ShowShortcut = true;

         mniEdit.MenuItems.Add(mniUndo);
         mniEdit.MenuItems.Add("-");
         mniEdit.MenuItems.Add(mniCut);
         mniEdit.MenuItems.Add(mniCopy);
         mniEdit.MenuItems.Add(mniPaste);
         mniEdit.MenuItems.Add("-");
         mniEdit.MenuItems.Add(mniFind);

         mniFormat.MenuItems.Add(mniFont);

         cmnuDocument.MenuItems.Add(mniCut.CloneMenu());
         cmnuDocument.MenuItems.Add(mniCopy.CloneMenu());
         cmnuDocument.MenuItems.Add(mniPaste.CloneMenu());
         cmnuDocument.MenuItems.Add("-");
         cmnuDocument.MenuItems.Add(mniFont.CloneMenu());
         cmnuDocument.Popup += new EventHandler(this.HandleContext);
         pnlView.ContextMenu = cmnuDocument;

         scrlbar.Dock = DockStyle.Right;
         scrlbar.Scroll += new ScrollEventHandler(this.ScrollView);
         scrlbar.Maximum = 10;

         this.Text = strName;
         this.Font = new Font("Courier New", 12);
         this.Size = new Size(800, 900);
         this.Name = "NetWord";
         this.WindowState = FormWindowState.Maximized;
         this.Menu = mnuMain;
         this.Closing += new CancelEventHandler(this.DocumentClosing);

         pnlView.AutoScroll = false;
         pnlView.BackColor = Color.DarkGray;
         pnlView.Width = this.Width - 16;
         pnlView.Height = this.Height - 50;
         pnlView.Location = new Point(0,0);
         pnlView.Font = fntCurrent;

         this.Controls.Add(scrlbar);
         this.Controls.Add(sbarMain);
         this.Controls.Add(pnlView);
         AddPage();
      }

      private void ScrollView(Object Sender, ScrollEventArgs e) {
         pnlView.Top = 0 - (e.NewValue * 30);
      }

      private void AddPage() {
         int intHeight;

         intHeight = objPageSettings.Bounds.Height - (int)(.96 * (objPageSettings.Margins.Top + objPageSettings.Margins.Bottom));

         Page objPage = new Page(this,objPageSettings,arrPages.Count+1);
         pnlView.Height = (arrPages.Count+1) * intHeight;
         pnlView.Controls.Add(objPage);
         scrlbar.Maximum = (30 * arrPages.Count+1) + 15;
         
         objPage.Location = new Point(50,intHeight * arrPages.Count + 5);
         objPage.Filled += new EventHandler(this.PageFilled);
         objPage.Enter += new EventHandler(this.PageGotFocus);
         objPage.SelectionChanged += new EventHandler(this.UpdateStatus);

         arrPages.Add(objPage);
         objPage.Focus();
      }

      private void PageFilled(Object Sender, EventArgs e) {
         if (intCurrentPage == arrPages.Count) {
            AddPage();
         } else {
            ((Page)arrPages[intCurrentPage]).Focus();
         }
      }

      private void PageGotFocus(Object Sender, EventArgs e) {
         intCurrentPage = ((Page)Sender).PageNumber;
      }

      private void UpdateStatus(Object Sender, EventArgs e) {
         Page tmpPage = (Page)arrPages[intCurrentPage-1];

         spnlLine.Text = "Page " + intCurrentPage.ToString() + "  Line " + tmpPage.LineIndex.ToString() + " Char " + tmpPage.CharIndex.ToString();
      }

      private void HandleContext(Object Sender, EventArgs e) {
         if (((Page)arrPages[intCurrentPage-1]).SelectionLength == 0) {
            cmnuDocument.MenuItems[0].Enabled = false;
            cmnuDocument.MenuItems[1].Enabled = false;
         } else {
            cmnuDocument.MenuItems[0].Enabled = true;
            cmnuDocument.MenuItems[1].Enabled = true;
         }
      }

      private void FileClicked(Object Sender, EventArgs e) {
         MenuItem mniTemp = (MenuItem)Sender;
         PrintDocument pd;
         String strText;

         switch (mniTemp.Text) {
            case "Save":
               FileInfo filTemp = new FileInfo(this.Text);
               if (filTemp.Extension == ".rtf") {
                  ((Page)arrPages[intCurrentPage-1]).SaveFile(this.Text, RichTextBoxStreamType.RichText);
               } else {
                  ((Page)arrPages[intCurrentPage-1]).SaveFile(this.Text, RichTextBoxStreamType.PlainText);
               }
               ((Page)arrPages[intCurrentPage-1]).Modified = false;
               break;
            case "Page Setup...":
               dlgPageSetup.PageSettings = objPageSettings;
               dlgPageSetup.ShowDialog();

               foreach (Page tmpPage in arrPages) {
                  tmpPage.UpdatePageSettings(objPageSettings);
               }
               
               break;
            case "Print Preview":
               strText = "";

               foreach (Page tmpPage in arrPages) {
                  strText += tmpPage.Text;
               }

               objReader = new StringReader(strText);

               pd = new PrintDocument();
               pd.DefaultPageSettings = objPageSettings;
               pd.PrintPage += new PrintPageEventHandler(this.PrintIt);

               PrintPreviewDialog dlgPrintPreview = new PrintPreviewDialog();
               dlgPrintPreview.Document = pd;
               
               dlgPrintPreview.ShowDialog();
               break;
            case "Print...":
               PrintDialog dlgPrint = new PrintDialog();
               dlgPrint.PrinterSettings = new PrinterSettings();
               if (dlgPrint.ShowDialog() == DialogResult.OK) {
                  strText = "";

                  foreach (Page tmpPage in arrPages) {
                     strText += tmpPage.Text;
                  }

                  objReader = new StringReader(strText);
  
                  pd = new PrintDocument();
                
                  pd.DefaultPageSettings = objPageSettings;
                  pd.PrintPage += new PrintPageEventHandler(this.PrintIt);
                  pd.Print();
               }
               break;
            case "Exit":
               this.Close();
               break;
         }
      }

      private void PrintIt(Object Sender, PrintPageEventArgs e) {
         Font fntPrint = this.Font;
         int count = 0; 
         float yPos = 0;
         float lpp = e.MarginBounds.Height / fntPrint.GetHeight(e.Graphics);
         float fltTopMargin = e.MarginBounds.Top;
         float fltLeftMargin = e.MarginBounds.Left;
         String strLine = null;
 
         while (count < lpp && ((strLine = objReader.ReadLine()) != null)) {
            yPos = fltTopMargin + (count * fntPrint.GetHeight(e.Graphics));
       
            e.Graphics.DrawString(strLine, fntPrint, Brushes.Black, fltLeftMargin, yPos, new StringFormat());
  
            count++;
         }
       
         if (strLine != null) {
            e.HasMorePages = true;
         } else {
            e.HasMorePages = false;
         }
      }

      private void EditClicked(Object Sender, EventArgs e) {
         MenuItem mniTemp = (MenuItem)Sender;
         Page tmpPage = (Page)arrPages[intCurrentPage-1];

         switch (mniTemp.Text) {
            case "Undo":
               tmpPage.Undo();
               break;
            case "Cut":
               if (tmpPage.SelectedRtf != "") {
                  tmpPage.Cut();
               }
               break;
            case "Copy":
               if (tmpPage.SelectedRtf != "") {
                  tmpPage.Copy();
               }
               break;
            case "Paste":
               tmpPage.Paste();
               break;
            case "Find...":
               FindDialog dlgFind = new FindDialog(this);
               dlgFind.Show();
               break;
         }
      }

      private void FormatClicked(Object Sender, EventArgs e) {
         MenuItem mniTemp = (MenuItem)Sender;

         switch (mniTemp.Text) {
            case "Font...":
               dlgFont.ShowColor = true;
               dlgFont.Font = this.Font;
               if (dlgFont.ShowDialog() == DialogResult.OK) {
                  fntCurrent = dlgFont.Font;
                  fntColor = dlgFont.Color;
                  this.Font = fntCurrent;
                  this.ForeColor = fntColor;
                  ((Page)arrPages[intCurrentPage - 1]).SetFont(fntCurrent, fntColor);
               }
               break;
         }
      }

      private void DocumentClosing(Object Sender, CancelEventArgs e) {
         if (((Page)arrPages[intCurrentPage-1]).Modified) {
            DialogResult dr = MessageBox.Show("Do you want to save the changes?", this.Name, MessageBoxButtons.YesNoCancel);

            if (dr == DialogResult.Yes) {
               FileClicked(mniSave, new EventArgs());
            } else if (dr == DialogResult.Cancel) {
               e.Cancel = true;
            }
         }
      }
   }
}